#!/bin/bash

# Variables for paths
PARSER=${PARSER:='parseAF'}
CAMUS=${CAMUS:='camus_mcs'}
SEMIASP=${SEMIASP:='semi.dl'}
CLINGO=${CLINGO:='clingo'}
# instance
INST=${INST:='test.dl'}
TMPDIMACS=${TMPDIMACS:=''}
TMPERR=${TMPERR:=''}
TMPMAP=${TMPMAP:=''}
TMPRANGE=${TMPRANGE:=''}
NUMBEROFARGS=${NUMBEROFARGS:=''}
NUMBEROFCLAUSES=${NUMBEROFCLAUSES:=''}
TMPCAMUS=${TMPCAMUS:=''}
TMPYVARS=${TMPYVARS:=''}
WORKINGDIR=${WORKINGDIR:=''}
TMPDIR=${TMPDIR:=''}
MODE=${MODE:=''}
ARG=${ARG:=''}

# create a bunch of tmp fifos, since may solver cannot usually read from stdin all the time
TMPDIR=$(mktemp -d)
trap 'rm -rf "$TMPDIR"' EXIT INT TERM HUP

mkfifo "$TMPDIR/TMPRANGE"
mkfifo "$TMPDIR/TMPCAMUS"

# the input for camus is constructed a bit more complicated through different processes, hence we use tmp files
TMPDIMACS=`mktemp -p $TMPDIR`
TMPERR=`mktemp -p $TMPDIR`
TMPYVARS=`mktemp -p $TMPDIR`
TMPMAP=`mktemp -p $TMPDIR`

usage() {
echo $0 "instance mode [arg]"
echo "instance AF in ASP format: (one per line)"
echo "           arg(x).   ... x is an argument"
echo "           att(x,y). ... x attacks y"
echo "mode ... {enum, cred, skept, allSkept}"
echo "           cred, skept require argument to query for"
echo "Example call: " $0 " instance.dl cred \"a\" "
}

INST=$1

# usage
if [ "$INST" == "" ];then
usage
exit 1
fi

if [ "$2" == "enum" ];then
MODE="-a"
elif [ "$2" == "cred" -a "$3" != "" ];then
MODE="-r"
ARG=$3
elif [ "$2" == "skept" -a "$3" != "" ];then
MODE="-w"
ARG=$3
elif [ "$2" == "allSkept" ];then
MODE="-b"
else
usage
exit 1
fi

WORKINGDIR=`dirname $0`

# fetch number of variables:
NUMBEROFARGS=`grep "arg" $INST | wc -l`

# parse instance to DIMACs, save mappings
$WORKINGDIR/$PARSER $INST > $TMPDIMACS 2> $TMPERR
grep "map" $TMPERR > $TMPMAP
grep -v "map" $TMPERR > $TMPDIR/TMPRANGE &
NUMBEROFCLAUSES=`wc -l $TMPDIMACS | awk '{print $1}'`

for ((i=1;i<=$NUMBEROFCLAUSES;i++))
do
echo $i 0 >> $TMPYVARS
done

cat $TMPDIR/TMPRANGE >> $TMPDIMACS
NUMBEROFCLAUSES=`wc -l $TMPDIMACS | awk '{print $1}'`

echo "p cnf "$NUMBEROFARGS $NUMBEROFCLAUSES > $TMPDIMACS.new && cat $TMPDIMACS >> $TMPDIMACS.new && mv $TMPDIMACS.new $TMPDIMACS

# call CAMUS

# debug information
#cat $TMPDIMACS > TMPDIMACS
#cat $TMPYVARS > TMPYVARS

if [ "$MODE" == "-a" ]; then
$WORKINGDIR/$CAMUS $TMPDIMACS -y $TMPYVARS $MODE | grep "Model" > $TMPDIR/TMPCAMUS &

cat $TMPDIR/TMPCAMUS | while read line; do 
    echo $line | grep -o "\-[0-9]*\|[0-9]*" | grep -v "\-" | awk '{print "semiCNF("$1")."}' | $WORKINGDIR/$CLINGO - $WORKINGDIR/$SEMIASP $TMPMAP | head -n 2 | tail -n 1
done

elif [ "$MODE" == "-b" ]; then
$WORKINGDIR/$CAMUS $TMPDIMACS -y $TMPYVARS $MODE | grep "Skeptically accepted" | grep -o "[0-9]*" | awk '{print "semiCNF("$1")."}' > $TMPDIR/TMPCAMUS &

cat $TMPDIR/TMPCAMUS | while read line; do 
    echo $line | $WORKINGDIR/$CLINGO - $WORKINGDIR/$SEMIASP $TMPMAP | head -n 2 | tail -n 1
done

else
ARG=`grep "map($ARG,[0-9]*)." $TMPMAP | awk -F"[(,)]" '{print $3}'`
$WORKINGDIR/$CAMUS $TMPDIMACS -y $TMPYVARS $MODE $ARG #| grep "Model" > $TMPDIR/TMPCAMUS &
fi
